#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "comandos.h"
#include "imagem.h"

/* NAO ALTERAR ESTE ARQUIVO */

int main() {
   /* ----------------------------------------------------------------------
      Declaraes e incio para verificao do uso da memria dinmica.
      ---------------------------------------------------------------------- */
   struct mallinfo info = mallinfo();
   int MemDinInicial = info.uordblks, MemDinFinal;
   /* ---------------------------------------------------------------------- */
   int acao, thr, x0, y0, x1, y1;
   FILE *fp = stdin;
   boolean fim = false;
   Image *img = NULL;
   char filename[1024];

   for (acao=getc(fp); acao!=EOF && !fim; acao=getc(fp)) {
     switch ( acao ) {
       /* Carrega a imagem na memoria */
       case 'r':
	 if (fscanf(fp, "%s", filename) != 1) {
	   fprintf(stderr, "Erro ao ler nome de arquivo.\n");
	   exit(-1);
	 }
	 if ( !comando_carrega_imagem(filename, &img) ) {
	   fprintf(stderr, "r: erro na leitura da imagem\n");
	 }
	 break;
       /* Imprime a imagem no formato PGM P2 */
       case 'p':
	 if (!comando_imprime_imagem(img)) {
	   fprintf(stderr, "p: erro na impressao da imagem\n");
	 }
	 break;
       /* Libera a imagem */
       case 'c':
	 if (!comando_libera_imagem(&img)) {
	   fprintf(stderr, "c: erro ao liberar imagem\n");
	 }
	 break;
       /* Imprime o histograma normalizado */
       case 'h':
	 if (!comando_imprime_histograma(img)) {
	   fprintf(stderr, "h: erro na impressao do histograma normalizado\n");
	 }
	 break;
       /* Imprime o histograma normalizado acumulado */
       case 'H':
	 if (!comando_imprime_histograma_acumulado(img)) {
	   fprintf(stderr, "H: erro na impressao do histograma normalizado acumulado\n");
	 }
	 break;
       /* Equaliza o histograma da imagem */
       case 'e':
	 if (!comando_equaliza_histograma(img)) {
	   fprintf(stderr, "e: erro na equalizacao do histograma\n");
	 }
	 break;
       /* Calcula o negativo da imagem */
       case 'n':
	 if (!comando_negativo_imagem(img)) {
	   fprintf(stderr, "n: erro ao calcular negativo da imagem\n");
	 }
	 break;
       /* Limiariza a imagem */
       case 't':
	 scanf("%d", &thr);
	 if (!comando_limiariza_imagem(img, thr)) {
	   fprintf(stderr, "t: erro ao limiarizar imagem\n");
	 }
	 break;
       /* Imprime o valor medio de brilho */
       case 'm':
	 if (!comando_media_brilho(img)) {
	   fprintf(stderr, "m: erro no calculo da media de brilho da imagem\n");
	 }
	 break;
       /* Recorta um retangulo (x0,y0), (x1,y1) da imagem */
       case 'o':
	 if (scanf("%d %d %d %d", &x0, &y0, &x1, &y1) != 4) {
	   fprintf(stderr, "Erro ao ler posicoes x0, y0, x1, y1.\n");
	 } else if (!comando_recorta_retangulo(&img, x0, y0, x1, y1)) {
	   fprintf(stderr, "o: erro ao recortar imagem.\n");
	 }
	 break;
       /* Sai do programa */
       case 'q':
	 fim = true;
	 break;

       /* Caracteres ignorveis */
       case ' ':
       case '\n':
       case '\t':
	 break;

       default:
	 fprintf(stdout, "Opo desconhecida: `%c\n", acao);
	 break;
     }
   }

   if (img != NULL) {
     DestroyImage(&img);
   }

   /* --------------------------------------------------------------------- */
   /* Verificao final do uso da memria dinmica                          */
   /* --------------------------------------------------------------------- */
   info = mallinfo();
   MemDinFinal = info.uordblks;
   if (MemDinInicial!=MemDinFinal)
      printf("\n\nMemria dinmica no foi totalmente liberada (%d, %d)\n",
	     MemDinInicial,MemDinFinal);
   /* --------------------------------------------------------------------- */
   return 0;
}
