#include <stdio.h>
#include <stdlib.h>
#include "atomos.h"

void ImprimeAtomosDeCadeia(char *cadeia) {
   SeparadorEstado s;
   SeparadorInicializa(&s, cadeia);
   Atomo *at;

   while ( (at = SeparadorProxAtomo(&s)) != NULL ) {
      printf("[");ImprimeAtomo(at);printf("] ");
      free(at);
   }
   printf("\n");
}

void SeparadorInicializa(SeparadorEstado *t, char *str) {
   t->pos = 0;
   t->str = str;
}

Atomo* SeparadorProxAtomo(SeparadorEstado *t) {
   int r, n;
   char buf[4];
   double cte;
   Atomo *at;

   sscanf(t->str+t->pos," %n", &n); // ignora espaos em branco
   t->pos += n;

   // tenta ler uma varivel
   r = sscanf(t->str+t->pos,"%1[a-z]%n", buf, &n);
   if ( r > 0 ) {
      t->pos += n;
      at = malloc(sizeof(*at));
      at->tipo = variavel;
      at->val.var = buf[0];
      return at;
   }

   // tenta ler um operador
   r = sscanf(t->str+t->pos, "%1[+-*/^]%n", buf, &n);
   if ( r > 0 ) {
      t->pos += n;
      at = malloc(sizeof(*at));
      at->tipo = operador;
      at->val.op = buf[0];
      return at;
   }

   // tenta ler um parenteses
   r = sscanf(t->str+t->pos, "%1[()]%n", buf, &n);
   if ( r > 0 ) {
      t->pos += n;
      at = malloc(sizeof(*at));
      at->tipo = parenteses;
      at->val.par = buf[0];
      return at;
   }

   // tenta ler uma constante
   r = sscanf(t->str+t->pos, "%lf%n", &cte, &n);
   if ( r > 0 ) {
      t->pos += n;
      at = malloc(sizeof(*at));
      at->tipo = constante;
      at->val.cte = cte;
      return at;
   }

   return NULL;
}


void ImprimeAtomo(Atomo* atomo) {
   switch (atomo->tipo) {
      case operador:
	 printf("%c", atomo->val.op);
	 break;
      case variavel:
	 printf("%c", atomo->val.var);
	 break;
      case constante:
	 printf("%.10g", atomo->val.cte);
	 break;
      case parenteses:
	 printf("%c", atomo->val.par);
	 break;
      default:
	 printf("ImprimeAtomo: tipo inesperado %d\n", atomo->tipo);
	 exit(3);
   }
}
