/* MC202 - Estruturas de dados - 2. semestre de 2005.
 * Funes para a montagem da rvore de Huffman. */

#ifndef ARVHUFF_H
#define ARVHUFF_H

#define N_CHARS 256 /* nmero de caracteres possveis */

typedef struct no {
  unsigned char c; /* caracter armazenado no n */
  int freq; /* frequencia do caracter */
  struct no *esq, *dir; /* filhos esquerdo e direito */
} No;


/* Rotina auxiliar para comparao das freqncias das sub-rvores.
 * Retorna um valor menor que zero se arv1->freq < arv2->freq ou um
 * valor maior que zero se arv1->freq > arv2->freq.  Caso as
 * freqncias sejam iguais, retorna um valor menor do que zero caso
 * arv1 contenha um caracter de valor ASCII menor que arv2, ou
 * retorna um valor maior do que zero caso contrrio. */
int compara_no(No *arv1, No *arv2);

/* Funo que cria um n vazio. */
No *cria_no_vazio();

/* Retorna um vetor de tamanho N_CHARS com a freqncia dos caracteres do
 * arquivo fornecido. O ndice no vetor representa o valor ASCII de um
 * caracter. */
int *conta_frequencia_arquivo(FILE *arq);

/* Faz a unio de duas sub-rvores. O novo n deve conter o caracter de
 * menor valor ASCII e a soma das freqncias das sub-rvores. A sub-rvore
 * de menor freqncia deve ficar  esquerda. Em caso de empate, a rvore
 * com o caracter de menor valor ASCII deve ficar  esquerda. */
No *merge_no(No *arv1, No *arv2);

/* Gera a rvore a partir de um arquivo. */
No *gera_arvore_arquivo(char *nomearq);

/* Libera toda a memria ocupada pela rvore. */
void destroi_arvore(No *arv);

/* Retorna um vetor de strings de tamanho N_CHARS com a codificao de
 * Huffman dos caracteres. O ndice no vetor representa o valor ASCII de
 * um caracter. Quando um caracter no possuir codificao, sua posio
 * no vetor deve conter NULL. arv aponta para a raiz da rvore. */
char **gera_tabela_arvore(No *arv);

/* Libera toda a memria ocupada pela tabela. */
void destroi_tabela(char **tabela);

/* Imprime a tabela de codificao. A primeira linha deve ser o nmero de
 * codificaes. */
void imprime_tabela(char **tabela);

/* Insere o caracter c com codificao codif na rvore apontada por arv. */
void insere_arvore(No *arv, unsigned char c, char *codif);

#endif
