#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <values.h>
#include "pagina_gab.h"


bool InserePagina_gab(Grafo *grafo, int indice, char *url) {
  if ((grafo == NULL) || (indice >= grafo->npaginas))
    return false;
  Pagina *p = &(grafo->pagina[indice]);
  p->url = (char *) malloc((strlen(url) + 1) * sizeof(char));
  strcpy(p->url, url);
  p->link = NULL;
  return true;
}


bool InsereLink_gab(Grafo *grafo, int indice, int destino) {
  if ((grafo == NULL) || (indice >= grafo->npaginas))
    return false;

  Pagina *p = &(grafo->pagina[indice]);

  Link *atual = p->link;
  Link **ant = &(p->link);
  while ((atual != NULL) && (atual->indice < destino)) { /* busca posicao. */
    ant = &(atual->prox);
    atual = atual->prox;
  }

  if ((atual != NULL) && (atual->indice == destino)) /* link repetido. */
     return false;

  /* cria um novo link. */
  Link *l = (Link *) malloc(sizeof(Link));
  l->indice = destino;
  l->prox = atual;
  *ant = l;
  return true;
}


void DestroiGrafo_gab(Grafo **grafo) {
  if (*grafo == NULL)
    return;

  Grafo *g = *grafo;

  for (int i = 0; i < g->npaginas; i++) { /* libera url e links. */
    Link *l = g->pagina[i].link;
    while (l != NULL) { /* libera links. */
      Link *aux = l;
      l = l->prox;
      free(aux);
    }
    free(g->pagina[i].url); /* libera url. */
  }

  free(g->pagina);
  free(g);
  *grafo = NULL;
}


char *BuscaPaginaMaisPopular_gab(Grafo *grafo, int *nlinks) {
  if (grafo == NULL)
    return NULL;

  int *contador = (int *) calloc(grafo->npaginas, sizeof(int));

  /* conta o numero de referencias para cada pagina. */
  for (int i = 0; i < grafo->npaginas; i++) {
    Link *l = grafo->pagina[i].link;
    while (l != NULL) {
      contador[l->indice]++;
      l = l->prox;
    }
  }

  /* busca o indice com a maior contagem. */
  int maiori=0;
  *nlinks = -1;
  for (int i = 0; i < grafo->npaginas; i++) {
    if (contador[i] > *nlinks) {
      *nlinks = contador[i];
      maiori = i;
    }
  }

  free(contador);

  return grafo->pagina[maiori].url;
}


int BuscaCaminhoMinimo_gab(Grafo *grafo, char *pi, char *pf) {
   int nv = grafo->npaginas;
   int *custo = (int *) malloc(nv*sizeof(int));
   Heap *h = cria_heap(nv, custo);
   int vi=-1, vf=-1;

   /* procura os indices das urls. */
   for (int i = 0; i < grafo->npaginas; i++) {
      if (strcmp(grafo->pagina[i].url, pi) == 0)
	 vi = i;
      if (strcmp(grafo->pagina[i].url, pf) == 0)
	 vf = i;
   }
   if ((vi == -1) || (vf == -1)) {
      printf("### Erro, pagina inexistente ###\n");
      return -1;
   }

   custo[vi] = 0;
   sobe_heap(h, custo, vi);

   int num = 0;

   while ( ! heap_vazio(h) ) {
      num++;
      int vmenor = remove_menor_heap(h, custo);

      for (Link *p = grafo->pagina[vmenor].link; p != NULL; p = p->prox) {
	 if (esta_no_heap(h, p->indice) && custo[p->indice] > (custo[vmenor]+1)){
	    custo[p->indice] = custo[vmenor]+1;
	    sobe_heap(h, custo, p->indice);
	 }
      }
   }

   int r = custo[vf];

   free(custo);
   destroi_heap(&h);
   return (r==INFINITO?-1:r);
}


bool ExisteCicloAux_gab(Grafo *grafo, int indice, bool *visitado) {
  /* percorre adjacentes. */
  for (Link *l = grafo->pagina[indice].link; l != NULL; l = l->prox)
    if (!visitado[l->indice]) {
      /* pgina no visitada - marca e visita. */
      visitado[l->indice] = true;
      if (ExisteCicloAux_gab(grafo, l->indice, visitado))
	return true;
      visitado[l->indice] = false;
    } else {
      /* encontrou uma pagina j visitada. */
      return true;
    }
  return false;
}

bool ExisteCiclo_gab(Grafo *grafo) {

  /* vetor de paginas visitadas. */
  bool *visitado = (bool *) calloc(grafo->npaginas, sizeof(bool));

  /* percorre todas as paginas. */
  for (int i = 0; i < grafo->npaginas; i++) {
    visitado[i] = true;
    if (ExisteCicloAux_gab(grafo, i, visitado)) {
      free(visitado);
      return true;
    }
    visitado[i] = false;
  }

  free(visitado);
  return false;
}


void ImprimePaginasAlcancaveis_gab(Grafo *grafo, char *pi) {

  /* fila de paginas. */
  int *fila = (int *) malloc(grafo->npaginas * sizeof(int));
  /* inicio da fila, final da fila. */
  int filai = 0, filaf = 0;
  /* vetor de paginas visitadas. */
  bool *visitado = (bool *) calloc(grafo->npaginas, sizeof(bool));
  /* indice da pagina inicial. */
  int inicio = 0;

  /* procura o indice da url inicial. */
  while ((inicio < grafo->npaginas)
	 && (strcmp(grafo->pagina[inicio].url, pi) != 0))
    inicio++;
  if (inicio == grafo->npaginas) {
    printf("### Erro, pagina inexistente ###\n");
    return;
  }
  
  /* coloca a pagina inicial na fila. */
  visitado[inicio] = true;
  fila[filaf++] = inicio;

  /* busca em largura, marcando todas as paginas alcancaveis. */
  while (filai != filaf) {
    /* retira a primeira pagina da fila e percorre adjacentes. */
    for (Link *l = grafo->pagina[fila[filai++]].link; l != NULL; l = l->prox)
      if (!visitado[l->indice]) {
	visitado[l->indice] = true;
	fila[filaf++] = l->indice;
      }
  }

  printf("%s:", grafo->pagina[inicio].url);

  /* imprime as urls em ordem de indice */
  for (int i = 0; i < grafo->npaginas; i++) {
    if ((i != inicio) && visitado[i])
      printf(" %s", grafo->pagina[i].url);
  }
  printf("\n");

  free(visitado);
  free(fila);
}
