#include <stdio.h>
#include "comandos.h"
#include "polinomios.h"


/*--------------- Comando de leitura ---------------*/

/* L um polinmio do arquivo f na notao usual, e 
 * empilha o polinmio lido.
 * Abaixo alguns exemplos de polinmios aceitos:
 * [10x^99+15]
 * [10 x^12 + 5.55 x^2 + 13.0 - 7 x^-1]
 * [10x + x]
 * [-x]
 * [x]
 * [55x^0]
 * [10x + x]
 * [50-x^-1]
 * Retorna true se o polinmio tem a sintaxe correta,
 * e false caso contrrio.
 *
 * J vem implementado. No modifique.
 */
boolean comando_le_polinomio( FILE *f, pilha *p ) {
   int r, s, t;
   double coef;
   int exp;
   char foo[2]="+", bar[2]="";
   boolean first=true; /* indica se estamos lendo o primeiro monmio */
   boolean erro=false;

   polinomio pol = polinomio_cria_nulo();

   while (true) {
      /* verifica se h sinal ou se chegou ao fim */
      s = fscanf(f, " %1[]+-]", foo);
      if (foo[0]==']') break; /* final do polinmio */
      if (!first&&s<=0) { erro=true; break; }/* monmios devem ser separados por +/- */
      if (s<=0) { s=1; foo[0] = '+'; } /* assume sinal de mais */

      /* garante que o prximo caractere pertence a [0-9x] */
      if ( fscanf(f, " %1[^0-9x]", bar) != 0 ) { erro=true; break; }

      /* tenta ler "c_i x ^ i" */
      r = fscanf(f, " %lf %1[x] ^ %d", &coef, bar, &exp);

      /* tenta ler monmio com coeficiente implcito 1.0 */
      if ( r==0 && (t = fscanf(f," %1[x] ^ %d", bar, &exp)) ) {
	 r = t+1; coef = 1.0;
      }

      /* se havia sinal de menos, nega o coeficiente */
      if ( foo[0]=='-' ) coef = -coef;

      if (r<=0) { erro=true; break; }
      /* r==1: coeficiente de x^0 lido
       * r==2: coeficiente de x lido
       * r==3: coeficiente e expoente de x lidos
       */
      if (r==1 || (r==3 && exp==0)) {
	 polinomio_adiciona_monomio(&pol, coef, 0);
      } else if (r==2 || (r==3 && exp==1)) {
	 polinomio_adiciona_monomio(&pol, coef, 1);
      } else if (r==3) {
	 polinomio_adiciona_monomio(&pol, coef, exp);
      }

      first = false;
   }

   if (first) { erro=true; } /* polinmio vazio */
   if (erro) {
      polinomio_libera(&pol);
      return false;
   }

   if (pilha_cheia(p)) return false;
   pilha_empilha(p, (void *)pol);

   return true;
}


/*--------------- Comandos de impresso ---------------*/

/* Imprime o polinmio no topo da pilha, sem alterar a pilha.
 * Retorna false se a pilha estiver vazia, true caso contrrio.
 */
boolean comando_p(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Imprime o polinmio no topo da pilha e desempilha.
 * Retorna false se a pilha estiver vazia, true caso contrrio.
 */
boolean comando_P(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Imprime todo o contedo da pilha, sem alterar nada.
 * Comea a impresso a partir do topo da pilha.
 * No imprime nada se a pilha estiver vazia.
 * Retorna true.
 */
boolean comando_f(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}


/*--------------- Comandos aritmticos ---------------*/

/* Desempilha dois polinmios da pilha, executa a soma,
 * e empilha o resultado.
 * Retorna false se no for possvel (mantm a pilha como estava), true c.c.
 */
boolean comando_soma(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Desempilha dois polinmios da pilha, subtrai o primeiro desempilhado
 * do segundo, e empilha o resultado.
 * Retorna false se no for possvel (mantm a pilha como estava), true c.c.
 */
boolean comando_subtracao(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Desempilha dois polinmios da pilha, executa a multiplicao, e empilha
 * o resultado.
 * Retorna false se no for possvel (mantm a pilha como estava), true c.c.
 */
boolean comando_multiplicacao(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Desempilha um polinmio da pilha, e empilha a negao deste polinmio.
 * Retorna false se no for possvel (mantm a pilha como estava), true c.c.
 */
boolean comando_negativo(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}


/*--------------- Comandos de manipulao da pilha ---------------*/

/* Limpa a pilha, tornando-a vazia.
 * Retorna true.
 */
boolean comando_c(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Duplica o polinmio que est no topo da pilha, empilhando outra 
 * cpia dele. Assim, ``[x+1]d*p computa (x+1)^2.
 * Retorna false se no for possvel, mantendo a pilha como estava.
 */
boolean comando_d(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}

/* Troca a ordem na pilha dos dois primeiros polinmios a partir do topo.
 * Retorna false se no for possvel, mantendo a pilha como estava.
 */
boolean comando_r(pilha *p) {
   // COMPLETAR
   return true; // PROVISRIO
}


/*--------------- Outros comandos ---------------*/

/* Interpreta o resto da linha como comentrio.
 * J vem implementado. No modifique.
 */
boolean comando_comentario(FILE *f) {
   int c;
   do { c = getc(f); } while ( c != '\n' );
   return true;
}
