#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "comandos.h"
#include "pilha.h"
#include "polinomios.h"

#define MAX_PILHA 1024

int main() {
   /* ----------------------------------------------------------------------
      Declaraes e incio para verificao do uso da memria dinmica.
      ---------------------------------------------------------------------- */
   struct mallinfo info = mallinfo();
   int MemDinInicial = info.uordblks, MemDinFinal;
   /* ---------------------------------------------------------------------- */
   int acao;
   FILE *fp = stdin;
   boolean fim = false;
   pilha p;

   p = pilha_cria_vazia(MAX_PILHA);

   for (acao=getc(fp); acao!=EOF && !fim; acao=getc(fp)) {
      switch ( acao ) {
	 /* Comando de leitura */
	 case '[':
	    if ( !comando_le_polinomio(fp, &p) ) {
	       fprintf(stdout,"[: erro na leitura do polinmio\n");
	    }
	    break;

	 /* Comandos de impresso */
	 case 'p':
	    if ( !comando_p(&p) ) {
	       fprintf(stdout,"p: pilha vazia\n");
	    }
	    break;
	 case 'P':
	    if ( !comando_P(&p) ) {
	       fprintf(stdout,"P: pilha vazia\n");
	    }
	    break;
	 case 'f':
	    if ( !comando_f(&p) ) {
	       fprintf(stdout,"f: erro ao imprimir a pilha\n");
	    }
	    break;

	 /* Comandos aritmticos */
	 case '+':
	    if ( !comando_soma(&p) ) {
	       fprintf(stdout,"+: erro durante soma\n");
	    }
	    break;
	 case '-':
	    if ( !comando_subtracao(&p) ) {
	       fprintf(stdout,"-: erro durante subtrao\n");
	    }
	    break;
	 case '*':
	    if ( !comando_multiplicacao(&p) ) {
	       fprintf(stdout,"*: erro durante multiplicao\n");
	    }
	    break;
	 case '~':
	    if ( !comando_negativo(&p) ) {
	       fprintf(stdout,"~: erro durante negativo\n");
	    }
	    break;

	 /* Comandos de manipulao da pilha */
	 case 'c':
	    if ( !comando_c(&p) ) {
	       fprintf(stdout,"c: erro ao limpar a pilha\n");
	    }
	    break;
	 case 'd':
	    if ( !comando_d(&p) ) {
	       fprintf(stdout,"d: erro ao duplicar o topo da pilha\n");
	    }
	    break;
	 case 'r':
	    if ( !comando_r(&p) ) {
	       fprintf(stdout,
		       "r: erro ao trocar os dois valores do topo da pilha\n");
	    }
	    break;

	 /* Outros comandos */
	 case '#':
	    if ( !comando_comentario(fp) ) {
	       fprintf(stdout, "#: erro com comentrio\n");
	    }
	    break;
	 case 'q':
	    fim = true;
	    break;

	 /* Caracteres ignorveis */
	 case ' ':
	 case '\n':
	 case '\t':
	    break;

	 default:
	    fprintf(stdout, "Opo desconhecida: `%c\n", acao);
	    break;
      }
   }

   while (!pilha_vazia(&p)) {
      polinomio pol = (polinomio)pilha_desempilha(&p);
      polinomio_libera(&pol);
   }
   pilha_libera(&p);

   /* --------------------------------------------------------------------- */
   /* Verificao final do uso da memria dinmica                          */
   /* --------------------------------------------------------------------- */
   info = mallinfo();
   MemDinFinal = info.uordblks;
   if (MemDinInicial!=MemDinFinal)
      printf("\n\nMemria dinmica no foi totalmente liberada (%d, %d)\n",
	     MemDinInicial,MemDinFinal);
   /* --------------------------------------------------------------------- */
   return 0;
}
