#include <stdlib.h>
#include "pilha.h"

struct pilha {
   int tamanho_max;
   int tamanho;
   void* *elem; /* vetor de void* */
};


/* Cria e retorna uma pilha vazia com o tamanho especificado.
 */
pilha pilha_cria_vazia(int tamanho) {
   pilha p;
   p = (pilha)malloc(sizeof(struct pilha));
   p->tamanho_max = tamanho;
   p->tamanho = 0;
   p->elem = (void *)malloc(tamanho*sizeof(void *));
   return p;
}

/* Retorna o nmero mximo de elementos que a pilha p suporta.
 */
int pilha_tamanho_max(pilha *p) {
   return (*p)->tamanho_max;
}

/* Retorna o nmero de elementos que a pilha p possui.
 */
int pilha_tamanho(pilha *p) {
   return (*p)->tamanho;
}

/* Empilha um elemento representado por um apontador genrico
 * do tipo void* na pilha p.
 */
void pilha_empilha(pilha *p, void *elem) {
   if (pilha_cheia(p)) { return; }
   (*p)->elem[(*p)->tamanho++] = elem;
}

/* Desempilha um elemento da pilha p, retornando seu endereo.
 */
void *pilha_desempilha(pilha *p) {
   if (pilha_vazia(p)) return NULL;
   return (*p)->elem[--(*p)->tamanho];
}

/* Retorna o elemento do topo da pilha p, sem desempilh-lo.
 */
void *pilha_topo(pilha *p) {
   return (*p)->elem[(*p)->tamanho-1];
}

/* Retorna true se a pilha est vazia, false caso contrrio.
 */
boolean pilha_vazia(pilha (*p)) {
   return (*p)->tamanho == 0;
}

/* Retorna true se a pilha est cheia, false caso contrrio.
 */
boolean pilha_cheia(pilha *p) {
   return (*p)->tamanho==(*p)->tamanho_max;
}

/* Libera toda memria dinmica alocada pela pilha p.
 * No libera a memria alocada pelos elementos inseridos na pilha
 * (Isso  responsabilidade do chamador).
 */
void pilha_libera(pilha *p) {
   free((*p)->elem);
   free((*p));
   *p = NULL;
}
