/* MC202 - Estruturas de dados - 2. semestre de 2005.
 * Implementao de Heap para cdigo de Huffman. */

#include <stdlib.h>
#include <stdio.h>
#include "heap.h"

struct heap {
   /* pos[v]  a posio onde se encontra o vrtice v em vert */
   int *pos;

   /* vert  um heap, ordenado pela chave dist, fornecida nas
      chamadas das funes. Dado um vrtice v, temos:
      - dist[v]  a distncia atual at o vrtice v.
      - pos[v]  a posio onde se encontra v no vetor vert.
      - vert[pos[v]] == v
   */
   int *vert;
   int nelem;
};


int pai(int i) { return (i - 1) / 2; }
int filho_esq (int i) { return 2 * i + 1; }
int filho_dir (int i) { return 2 * i + 2; }

int existe_pai(int i) { return i > 0; }
int existe_filho_esq (int i, int n) { return (2 * i + 1) < n; }
int existe_filho_dir (int i, int n) { return (2 * i + 2) < n; }

Heap *cria_heap(int nvertices, int *dist) {
  Heap *heap = malloc(sizeof(Heap));
  heap->vert = malloc(nvertices*sizeof(int));
  heap->pos = malloc(nvertices*sizeof(int));
  for (int i=0; i<nvertices; ++i) {
     dist[i] = INFINITO;
     heap->vert[i] = i;
     heap->pos[i] = i;
  }
  heap->nelem = nvertices;
  return heap;
}

void destroi_heap(Heap **heap) {
   free((*heap)->vert);
   free((*heap)->pos);
   free(*heap);
   *heap = NULL;
}

void sobe_heap(Heap *heap, int *dist, int vert) {

   int filho = heap->pos[vert];
   int distvert = dist[vert];

   int ppai = pai(filho);

   while ( existe_pai(filho) && dist[heap->vert[ppai]] > distvert ) {
      int v = heap->vert[ppai];
      heap->pos[v] = filho;
      heap->vert[filho] = v;
      filho = ppai;
      ppai = pai(filho);
   }
   heap->pos[vert] = filho;
   heap->vert[filho] = vert;
}


void desce_heap(Heap *heap, int *dist, int pos) {

   int pai = pos;
   int vert = heap->vert[pai];
   int distvert = dist[vert];
   int filho = filho_esq(pai);

   while ( filho <= heap->nelem-1 ) {
      if ( (filho <= heap->nelem - 2) && 
	   (dist[heap->vert[filho]] > dist[heap->vert[filho+1]]) ) {
	 filho++; // pega o irmo da direita, porque  menor
      }

      if ( distvert > dist[heap->vert[filho]] ) {
	 int v = heap->vert[filho];
	 heap->pos[v] = pai;
	 heap->vert[pai] = v;
	 pai = filho;
	 filho = filho_esq(pai);
      } else {
	 break;
      }
   }
   heap->pos[vert] = pai;
   heap->vert[pai] = vert;
}

int esta_no_heap(Heap *heap, int v) {
   return heap->pos[v] >= 0;
}

int heap_vazio(Heap *heap) {
   return heap->nelem == 0;
}


int remove_menor_heap(Heap *heap, int *dist) {
   int ret;
   if ( heap->nelem > 0 ) {
      ret = heap->vert[0];
      heap->pos[ret] = -1; /* indica que no est no heap */

      heap->nelem--;
      if ( heap->nelem > 0 ) {
	 heap->vert[0] = heap->vert[heap->nelem];
	 heap->pos[heap->vert[0]] = 0;
	 desce_heap(heap, dist, 0);
      }
      return ret;
   }
   return -1;
}
