#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pagina.h"
#include "heap.h"
#ifndef NOGAB
#include "pagina_gab.h"
#endif

Funcoes funcao_testada;


/*
 * Cria a estrutura Grafo, alocando o vetor de pginas web com
 * tamanho 'npaginas'. J vem implementada.
 */
Grafo *CriaGrafo(int npaginas) {
  Grafo *grafo = (Grafo *) malloc(sizeof(Grafo));
  if (grafo == NULL) {
    printf("### Erro ao alocar memoria. ###\n");
    exit(-1);
  }
  grafo->pagina = (Pagina *) malloc(npaginas * sizeof(Pagina));
  if (grafo->pagina == NULL) {
    printf("### Erro ao alocar memoria. ###\n");
    exit(-1);
  }
  grafo->npaginas = npaginas;
  for (int i = 0; i < npaginas; i++) {
    grafo->pagina[i].url = NULL;
    grafo->pagina[i].link = NULL;
  }
  return grafo;
}


/*
 * Imprime os dados do grafo. J vem implementada.
 */
void ImprimeGrafo(Grafo *grafo) {
  if (grafo == NULL) {
    printf("=== Grafo nulo! ===\n");
    return;
  }
  printf("=== Pagina : Links ===\n");
  for (int i = 0; i < grafo->npaginas; i++) {
    printf("%s :", grafo->pagina[i].url);
    for (Link *l = grafo->pagina[i].link; l != NULL; l = l->prox)
      printf(" %s", grafo->pagina[l->indice].url);
    printf("\n");
  }
  printf("=== Total: %d paginas web ===\n", grafo->npaginas);
}


/*
 * Insere uma pgina web no grafo. 'indice'  o ndice da pgina web
 * no vetor e 'url'  uma string com a URL da pgina.
 * Inicialmente a pgina no possui links.
 * ATENO: A string apontada por 'url' poder ser modificada aps a
 *          chamada desta funo.
 * Retorna true se a pgina foi inserida com sucesso, ou false c.c.
 */
bool InserePagina(Grafo *grafo, int indice, char *url) {
#ifndef NOGAB
  if ( funcao_testada != f_InserePagina )
    return InserePagina_gab(grafo, indice, url);
#endif
  /* COMPLETAR A PARTIR DAQUI */

  return false; /* PROVISORIO */
}


/*
 * Insere um link em uma pgina web. 'indice'  o ndice da pgina web
 * e 'destino'  o ndice da pgina web destino.
 * Links repetidos devem ser desconsiderados, e a lista ligada de links
 * deve ficar em ordem crescente de ndices.
 * Retorna true se o link foi inserido com sucesso, ou false c.c.
 */
bool InsereLink(Grafo *grafo, int indice, int destino) {
#ifndef NOGAB
  if ( funcao_testada != f_InsereLink )
    return InsereLink_gab(grafo, indice, destino);
#endif
  /* COMPLETAR A PARTIR DAQUI */

  return false; /* PROVISORIO */
}


/*
 * Libera toda a memria ocupada pelo grafo.
 */
void DestroiGrafo(Grafo **grafo) {
#ifndef NOGAB
  if ( funcao_testada != f_DestroiGrafo ) {
    DestroiGrafo_gab(grafo);
    return;
  }
#endif
  /* COMPLETAR A PARTIR DAQUI */
}


/*
 * Busca a pgina web mais popular, isto , a que recebe mais links.
 * Retorna a URL da pgina web mais popular, e coloca na varivel
 * nlinks o nmero de links que levam  pgina web.
 * Em caso de empate, deve ser retornada a pgina com o menor ndice.
 */
char *BuscaPaginaMaisPopular(Grafo *grafo, int *nlinks) {
#ifndef NOGAB
  if ( funcao_testada != f_BuscaPaginaMaisPopular )
    return BuscaPaginaMaisPopular_gab(grafo, nlinks);
#endif
  /* COMPLETAR A PARTIR DAQUI */

  return NULL; /* PROVISORIO */
}


/*
 * Verifica a existncia de ciclos no grafo.
 * Retorna true se existir ciclo no grafo, ou false c.c.
 */
bool ExisteCiclo(Grafo *grafo) {
#ifndef NOGAB
  if ( funcao_testada != f_ExisteCiclo )
    return ExisteCiclo_gab(grafo);
#endif
  /* COMPLETAR A PARTIR DAQUI */

  return false; /* PROVISORIO */
}


/*
 * Imprime sem repetio todas as pginas que podem ser alcanadas
 * a partir de uma pgina 'pi'.
 * A sada deve seguir o formato "urli: url1 url2 url3 url4 ...",
 * onde 'urli'  a URL da pgina inicial, e as URLs seguintes so
 * as pginas que podem ser alcanadas, ordenadas pelo ndice das
 * pginas.
 */
void ImprimePaginasAlcancaveis(Grafo *grafo, char *pi) {
#ifndef NOGAB
  if ( funcao_testada != f_ImprimePaginasAlcancaveis ) {
    ImprimePaginasAlcancaveis_gab(grafo, pi);
    return;
  }
#endif
  /* COMPLETAR A PARTIR DAQUI */

}


/*
 * Encontra o menor nmero de links para ir da pgina 'pi' para a
 * pgina 'pf'.
 * Se 'pi' for igual a 'pf', o nmero de links deve ser zero.
 * Se no existir caminho de 'pi' a 'pf', a funo deve retornar -1.
 * Deve usar o heap, j implementado em heap.c.
 */
int BuscaCaminhoMinimo(Grafo *grafo, char *pi, char *pf) {
#ifndef NOGAB
  if ( funcao_testada != f_BuscaCaminhoMinimo )
    return BuscaCaminhoMinimo_gab(grafo, pi, pf);
#endif
  /* COMPLETAR A PARTIR DAQUI */

  return 0; /* PROVISORIO */
}
