#ifndef PAGINA_H
#define PAGINA_H

/** pagina.h **/

typedef enum {
  f_InserePagina, f_InsereLink, f_DestroiGrafo,
  f_BuscaPaginaMaisPopular, f_ExisteCiclo,
  f_ImprimePaginasAlcancaveis, f_BuscaCaminhoMinimo
} Funcoes;

typedef enum {false, true} bool;

typedef struct link { /* Estrutura que representa um link para outra pgina. */
  int indice; /* ndice da pgina destino. */
  struct link *prox; /* Apontador para o prximo link. */
} Link;

typedef struct { /* Estrutura que representa uma pgina web. */
  char *url; /* URL da pgina. */
  Link *link; /* Lista ligada de links da pgina. */
} Pagina;

typedef struct { /* Estrutura que representa o grafo de pginas web. */
  Pagina *pagina; /* Vetor de pginas web. */
  int npaginas; /* Total de pginas web no grafo. */
} Grafo;


/*
 * Cria a estrutura Grafo, alocando o vetor de pginas web com
 * tamanho 'npaginas'. J vem implementada.
 */
Grafo *CriaGrafo(int npaginas);


/*
 * Imprime os dados do grafo. J vem implementada.
 */
void ImprimeGrafo(Grafo *grafo);


/*
 * Insere uma pgina web no grafo. 'indice'  o ndice da pgina web
 * no vetor e 'url'  uma string com a URL da pgina.
 * Inicialmente a pgina no possui links.
 * ATENO: A string apontada por 'url' poder ser modificada aps a
 *          chamada desta funo.
 * Retorna true se a pgina foi inserida com sucesso, ou false c.c.
 */
bool InserePagina(Grafo *grafo, int indice, char *url);


/*
 * Insere um link em uma pgina web. 'indice'  o ndice da pgina web
 * e 'destino'  o ndice da pgina web destino.
 * Links repetidos devem ser desconsiderados, e a lista ligada de links
 * deve ficar em ordem crescente de ndices.
 * Retorna true se o link foi inserido com sucesso, ou false c.c.
 */
bool InsereLink(Grafo *grafo, int indice, int destino);


/*
 * Libera toda a memria ocupada pelo grafo.
 */
void DestroiGrafo(Grafo **grafo);


/*
 * Busca a pgina web mais popular, isto , a que recebe mais links.
 * Retorna a URL da pgina web mais popular, e coloca na varivel
 * nlinks o nmero de links que levam  pgina web.
 * Em caso de empate, deve ser retornada a pgina com o menor ndice.
 */
char *BuscaPaginaMaisPopular(Grafo *grafo, int *nlinks);


/*
 * Verifica a existncia de ciclos no grafo.
 * Retorna true se existir ciclo no grafo, ou false c.c.
 */
bool ExisteCiclo(Grafo *grafo);


/*
 * Imprime sem repetio todas as pginas que podem ser alcanadas
 * a partir de uma pgina 'pi'.
 * A sada deve seguir o formato "urli: url1 url2 url3 url4 ...",
 * onde 'urli'  a URL da pgina inicial, e as URLs seguintes so
 * as pginas que podem ser alcanadas, ordenadas pelo ndice das
 * pginas.
 */
void ImprimePaginasAlcancaveis(Grafo *grafo, char *pi);


/*
 * Encontra o menor nmero de links para ir da pgina 'pi' para a
 * pgina 'pf'.
 * Se 'pi' for igual a 'pf', o nmero de links deve ser zero.
 * Se no existir caminho de 'pi' a 'pf', a funo deve retornar -1.
 * Deve usar o heap, j implementado em heap.c.
 */
int BuscaCaminhoMinimo(Grafo *grafo, char *pi, char *pf);


#endif
